@@ -15,11 +15,11 @@ use MARC::Field;
 
 =head1 VERSION
 
-Version 0.10
+Version 0.13
 
 =cut
 
-$VERSION = '0.10';
+$VERSION = '0.13';
 $DEBUG = 0;
 
 use constant SUBFIELD_INDICATOR	=> "\x1F";
@@ -89,7 +89,7 @@ sub new($) {
 =head2 new_from_usmarc()
 
 Constructor for handling data from a USMARC file.  This function takes care of all
-the directory parsing & mangling.
+the tag directory parsing & mangling.
 
 Any warnings or coercions can be checked in the C<warnings()> function.
 
@@ -177,6 +177,55 @@ sub new_from_usmarc($) {
 	return $self;
 }
 
+=head2 new_from_microlif()
+
+Constructor for handling data from a microlif file.  This function takes care of all
+the directory parsing & mangling.
+
+Any warnings or coercions can be checked in the C<warnings()> function.
+
+Note that we are NOT expecting to get the trailing "`" mark at the end of the last line.
+
+=cut
+
+sub new_from_microlif($) {
+	my $class = shift;
+	my $text = shift;
+	my $self = new($class);
+
+	my @lines = split( /\n/, $text );
+	for my $line ( @lines ) {
+		# Ignore the file header if the calling program hasn't already dealt with it
+		next if $line =~ /^HDR/;
+
+		($line =~ s/^(\d\d\d|LDR)//) or
+			return _gripe( "Invalid tag number: ", substr( $line, 0, 3 ) );
+		my $tagno = $1;
+
+		($line =~ s/\^$//) or
+			$self->_warn( "Tag $tagno is missing a trailing caret." );
+
+		if ( $tagno eq "LDR" ) {
+			$self->leader( substr( $line, 0, LEADER_LEN ) );
+		} elsif ( $tagno < 10 ) {
+			$self->add_fields( $tagno, $line );
+		} else {
+			$line =~ s/^(.)(.)//;
+			my ($ind1,$ind2) = ($1,$2);
+			my @subfields;
+			my @subfield_data_pairs = split( /_(?=[a-z0-9])/, $line );
+			shift @subfield_data_pairs; # Leading _ makes an empty pair
+			for my $pair ( @subfield_data_pairs ) {
+				my ($subfield,$data) = (substr( $pair, 0, 1 ), substr( $pair, 1 ));
+				push( @subfields, $subfield, $data );
+			}
+			$self->add_fields( $tagno, $ind1, $ind2, @subfields );
+		}
+	} # for
+
+	return $self;
+}
+
 =head2 next_from_file(*FILEHANDLE)
 
 Reads the next record from the file handle passed in.
@@ -448,7 +497,7 @@ sub as_formatted() {
 		push( @lines, $field->as_formatted() );
 	}
 
-	return join( "\n", @lines );
+	return join( "\n", @lines, "" );
 } # as_formatted
 
 =head2 title()
@@ -686,11 +735,6 @@ Ideas are things that have been considered, but nobody's actually asked for.
 
 =over 4
 
-=item * Read from MicroLIF
-
-Create a C<new_from_microlif()> function that would handle the pretty 
-MicroLIF format.  Basically, a reverse of C<as_microlif()>.
-
 =item * Create multiple output formats.
 
 These could be ASCII, XML, or MarcMaker.
